@@ -7,8 +7,10 @@ from gpuhunt._internal.constraints import (
     KNOWN_AMD_GPUS,
     KNOWN_INTEL_ACCELERATORS,
     KNOWN_NVIDIA_GPUS,
+    KNOWN_TENSTORRENT_ACCELERATORS,
     KNOWN_TPUS,
     correct_gpu_memory_gib,
+    is_nvidia_superchip,
     matches,
 )
 from gpuhunt._internal.default import default_catalog, query
@@ -17,10 +19,12 @@ from gpuhunt._internal.models import (
     AcceleratorVendor,
     AMDGPUInfo,
     CatalogItem,
+    CPUArchitecture,
     IntelAcceleratorInfo,
     NvidiaGPUInfo,
     QueryFilter,
     RawCatalogItem,
+    TenstorrentAcceleratorInfo,
     TPUInfo,
 )
 
