@@ -1,12 +1,14 @@
 import copy
+import enum
 import importlib.resources
 import json
 import logging
 import re
 from collections import defaultdict, namedtuple
 from collections.abc import Iterable
+from concurrent.futures import ThreadPoolExecutor, as_completed
 from dataclasses import dataclass
-from typing import Optional
+from typing import Optional, cast
 
 import google.cloud.billing_v1 as billing_v1
 import google.cloud.compute_v1 as compute_v1
@@ -14,6 +16,7 @@ from google.cloud import tpu_v2
 from google.cloud.billing_v1 import CloudCatalogClient, ListSkusRequest
 from google.cloud.billing_v1.types.cloud_catalog import Sku
 from google.cloud.location import locations_pb2
+from typing_extensions import NotRequired, TypedDict
 
 from gpuhunt._internal.models import AcceleratorVendor, QueryFilter, RawCatalogItem
 from gpuhunt.providers import AbstractProvider
@@ -24,6 +27,7 @@ AcceleratorDetails = namedtuple("AcceleratorDetails", ["name", "memory"])
 # As of 2024-14-08, this mapping contains only Nvidia accelerators; update gpu_vendor
 # inferring code in fill_gpu_vendors_and_names() if a non-Nvidia accelerator is added
 accelerator_details = {
+    "nvidia-b200": AcceleratorDetails("B200", 180.0),
     "nvidia-a100-80gb": AcceleratorDetails("A100", 80.0),
     "nvidia-h100-80gb": AcceleratorDetails("H100", 80.0),
     "nvidia-h100-mega-80gb": AcceleratorDetails("H100", 80.0),
@@ -34,6 +38,7 @@ accelerator_details = {
     "nvidia-tesla-p4": AcceleratorDetails("P4", 8.0),
     "nvidia-tesla-t4": AcceleratorDetails("T4", 16.0),
     "nvidia-tesla-v100": AcceleratorDetails("V100", 16.0),
+    "nvidia-rtx-pro-6000": AcceleratorDetails("RTXPRO6000", 96.0),
 }
 CpuMemory = namedtuple("CpuMemory", ["cpu", "memory"])
 accelerator_limits = {
@@ -153,59 +158,58 @@ class GCPProvider(AbstractProvider):
         self.cloud_catalog_client = billing_v1.CloudCatalogClient()
 
     def list_preconfigured_instances(self) -> list[RawCatalogItem]:
-        instances = []
-        for region in self.regions_client.list(project=self.project):
-            for zone_url in region.zones:
-                zone = zone_url.split("/")[-1]
-                logger.info("Fetching instances for zone %s", zone)
-                for machine_type in self.machine_types_client.list(
-                    project=self.project, zone=zone
-                ):
-                    if (
-                        machine_type.deprecated.state
-                        == compute_v1.DeprecationStatus.State.DEPRECATED
-                    ):
+        def _list_zone_instances(zone: str) -> list[RawCatalogItem]:
+            zone_instances = []
+            logger.info("Fetching instances for zone %s", zone)
+            for machine_type in self.machine_types_client.list(project=self.project, zone=zone):
+                if machine_type.deprecated.state == compute_v1.DeprecationStatus.State.DEPRECATED:
+                    continue
+                gpu = None
+                if machine_type.accelerators:
+                    accelerator = machine_type.accelerators[0].guest_accelerator_type
+                    gpu = accelerator_details.get(accelerator)
+                    if gpu is None:
+                        logger.warning("Unknown accelerator type: %s", accelerator)
                         continue
-                    gpu = None
-                    if machine_type.accelerators:
-                        accelerator = machine_type.accelerators[0].guest_accelerator_type
-                        gpu = accelerator_details.get(accelerator)
-                        if gpu is None:
-                            logger.warning("Unknown accelerator type: %s", accelerator)
-                            continue
 
-                    instance = RawCatalogItem(
-                        instance_name=machine_type.name,
-                        location=zone,
-                        cpu=machine_type.guest_cpus,
-                        memory=round(machine_type.memory_mb / 1024, 1),
-                        gpu_count=(
-                            machine_type.accelerators[0].guest_accelerator_count if gpu else 0
-                        ),
-                        # gpu_name is canonicalized and gpu_vendor is set later
-                        # in fill_gpu_vendors_and_names(), for now we use AcceleratorType.name
-                        # as a name (it contains a vendor prefix like "nvidia-")
-                        gpu_name=(
-                            machine_type.accelerators[0].guest_accelerator_type if gpu else None
-                        ),
-                        gpu_vendor=None,
-                        gpu_memory=gpu.memory if gpu else None,
-                        price=None,
-                        spot=None,
-                        disk_size=None,
-                    )
-                    instances.append(instance)
+                instance = RawCatalogItem(
+                    instance_name=machine_type.name,
+                    location=zone,
+                    cpu=machine_type.guest_cpus,
+                    memory=round(machine_type.memory_mb / 1024, 1),
+                    gpu_count=(machine_type.accelerators[0].guest_accelerator_count if gpu else 0),
+                    # gpu_name is canonicalized and gpu_vendor is set later
+                    # in fill_gpu_vendors_and_names(), for now we use AcceleratorType.name
+                    # as a name (it contains a vendor prefix like "nvidia-")
+                    gpu_name=(
+                        machine_type.accelerators[0].guest_accelerator_type if gpu else None
+                    ),
+                    gpu_vendor=None,
+                    gpu_memory=gpu.memory if gpu else None,
+                    price=None,
+                    spot=None,
+                    disk_size=None,
+                )
+                zone_instances.append(instance)
+            return zone_instances
+
+        instances = []
+        futures = []
+        with ThreadPoolExecutor(max_workers=8) as ex:
+            for region in self.regions_client.list(project=self.project):
+                for zone_url in region.zones:
+                    zone = zone_url.split("/")[-1]
+                    futures.append(ex.submit(_list_zone_instances, zone))
+        for future in as_completed(futures):
+            instances.extend(future.result())
         return instances
 
     def add_gpus(self, instances: list[RawCatalogItem]):
-        n1_instances = defaultdict(list)
-        for instance in instances:
-            if instance.instance_name.startswith("n1-"):
-                n1_instances[instance.location].append(instance)
-
-        instances_with_gpus = []
-        for zone, zone_n1_instances in n1_instances.items():
+        def _list_zone_instances(
+            zone: str, zone_n1_instances: list[RawCatalogItem]
+        ) -> list[RawCatalogItem]:
             logger.info("Fetching GPUs for zone %s", zone)
+            zone_instances = []
             for accelerator in self.accelerator_types_client.list(project=self.project, zone=zone):
                 if accelerator.name not in accelerator_limits:
                     continue
@@ -217,7 +221,21 @@ class GCPProvider(AbstractProvider):
                         i.gpu_count = n
                         i.gpu_name = accelerator.name
                         i.gpu_memory = accelerator_details[accelerator.name].memory
-                        instances_with_gpus.append(i)
+                        zone_instances.append(i)
+            return zone_instances
+
+        n1_instances = defaultdict(list)
+        for instance in instances:
+            if instance.instance_name.startswith("n1-"):
+                n1_instances[instance.location].append(instance)
+
+        instances_with_gpus = []
+        futures = []
+        with ThreadPoolExecutor(max_workers=8) as ex:
+            for zone, zone_n1_instances in n1_instances.items():
+                futures.append(ex.submit(_list_zone_instances, zone, zone_n1_instances))
+        for future in as_completed(futures):
+            instances_with_gpus.extend(future.result())
         instances += instances_with_gpus
 
     def fill_prices(self, instances: list[RawCatalogItem]) -> list[RawCatalogItem]:
@@ -228,14 +246,17 @@ class GCPProvider(AbstractProvider):
 
         offers = []
         for instance in instances:
-            for spot in (False, True):
-                price = prices.get_instance_price(instance, spot)
+            for capacity_type in CapacityType:
+                price = prices.get_instance_price(instance, capacity_type)
                 if price is None:
                     continue
 
                 offer = copy.deepcopy(instance)
                 offer.price = round(price, 6)
-                offer.spot = spot
+                offer.spot = capacity_type is CapacityType.SPOT
+                cast(GCPCatalogItemProviderData, offer.provider_data)["is_dws_calendar_mode"] = (
+                    capacity_type is CapacityType.DWS_CALENDAR_MODE
+                )
                 offers.append(offer)
         return offers
 
@@ -259,6 +280,8 @@ class GCPProvider(AbstractProvider):
         offers = self.fill_prices(instances)
         self.fill_gpu_vendors_and_names(offers)
         offers.extend(get_tpu_offers(self.project))
+        set_flags(offers)
+        offers = add_legacy_g4_preview(offers)
         return sorted(offers, key=lambda i: i.price)
 
     @classmethod
@@ -266,32 +289,67 @@ class GCPProvider(AbstractProvider):
         return [
             i
             for i in offers
-            if any(
-                i.instance_name.startswith(family)
-                for family in [
-                    "e2-medium",
-                    "e2-standard-",
-                    "e2-highmem-",
-                    "e2-highcpu-",
-                    "m1-",
-                    "a2-",
-                    "g2-",
-                ]
+            if (
+                any(
+                    i.instance_name.startswith(family)
+                    for family in [
+                        "m4-",
+                        "c4-",
+                        "n4-",
+                        "h3-",
+                        "n2-",
+                        "e2-medium",
+                        "e2-standard-",
+                        "e2-highmem-",
+                        "e2-highcpu-",
+                        "m1-",
+                        "a2-",
+                        "g2-",
+                    ]
+                )
+                or (i.gpu_name and i.gpu_name not in ["K80", "P4"])
+            )
+            and not (
+                # Filter out on-demand offers that are not actually available on demand.
+                # https://cloud.google.com/compute/docs/accelerator-optimized-machines#consumption_option_availability_by_machine_type
+                i.spot == False
+                and not cast(GCPCatalogItemProviderData, i.provider_data).get(
+                    "is_dws_calendar_mode"
+                )
+                and (
+                    i.instance_name.startswith("a4x-")
+                    or i.instance_name.startswith("a4-")
+                    or i.instance_name.startswith("a3-ultragpu-")
+                    or (
+                        i.instance_name.startswith("a3-highgpu-")
+                        and (i.gpu_count is None or i.gpu_count < 8)
+                    )
+                )
             )
-            or (i.gpu_name and i.gpu_name not in ["K80", "P4"])
         ]
 
 
-RegionSpot = tuple[str, bool]
-PricePerRegionSpot = dict[RegionSpot, float]
+class GCPCatalogItemProviderData(TypedDict):
+    is_dws_calendar_mode: NotRequired[bool]
+
+
+class CapacityType(enum.Enum):
+    ON_DEMAND = enum.auto()
+    SPOT = enum.auto()
+    DWS_CALENDAR_MODE = enum.auto()
+
+
+RegionCapacityType = tuple[str, CapacityType]
+PricePerRegionCapacityType = dict[RegionCapacityType, float]
 
 
 class Prices:
     def __init__(self):
-        self.cpu: defaultdict[str, PricePerRegionSpot] = defaultdict(dict)
-        self.gpu: defaultdict[str, PricePerRegionSpot] = defaultdict(dict)
-        self.ram: defaultdict[str, PricePerRegionSpot] = defaultdict(dict)
-        self.local_ssd: PricePerRegionSpot = dict()
+        self.cpu: defaultdict[str, PricePerRegionCapacityType] = defaultdict(dict)
+        self.gpu: defaultdict[str, PricePerRegionCapacityType] = defaultdict(dict)
+        self.ram: defaultdict[str, PricePerRegionCapacityType] = defaultdict(dict)
+        self.local_ssd: PricePerRegionCapacityType = dict()
+        self.gpu_slice: defaultdict[str, PricePerRegionCapacityType] = defaultdict(dict)
 
     def add_skus(self, skus: Iterable[Sku]) -> None:
         for sku in skus:
@@ -314,15 +372,23 @@ class Prices:
                 self.add_storage_sku(sku)
 
     def add_compute_sku(self, sku: Sku) -> None:
-        r = re.match(
-            r"^(?:spot preemptible )?(.+) (gpu|ram|core)",
-            sku.description,
-            flags=re.IGNORECASE,
-        )
-        if not r:
+        if "(1 gpu slice)" in sku.description:
+            self.add_compute_gpu_slice_sku(sku)
             return
-        family, resource = r.groups()
-        resource = resource.lower()
+
+        if "RTX 6000 96GB" in sku.description:
+            family = "nvidia-rtx-pro-6000"
+            resource = "gpu"
+        else:
+            r = re.match(
+                r"^(?:spot preemptible )?(.+) (gpu|ram|core)",
+                sku.description,
+                flags=re.IGNORECASE,
+            )
+            if not r:
+                return
+            family, resource = r.groups()
+            resource = resource.lower()
 
         if resource == "gpu":
             family = family.replace(" ", "-").lower()
@@ -343,8 +409,7 @@ class Prices:
                     "A3Plus Instance": "a3-megagpu",
                 }.get(family, family)
 
-        price = sku.pricing_info[0].pricing_expression.tiered_rates[0].unit_price
-        price = price.units + price.nanos / 1e9
+        price = self._calculate_sku_price(sku)
         resource_prices = {
             "core": self.cpu,
             "gpu": self.gpu,
@@ -352,36 +417,67 @@ class Prices:
         }[resource]
         self._add_price(sku, resource_prices[family], price)
 
+    def add_compute_gpu_slice_sku(self, sku: Sku) -> None:
+        if sku.description.startswith(
+            "Spot Preemptible A4 Nvidia B200"
+        ) or sku.description.startswith("DWS Calendar Mode A4 Nvidia B200"):
+            gpu = "nvidia-b200"
+        else:
+            return
+        price = self._calculate_sku_price(sku)
+        self._add_price(sku, self.gpu_slice[gpu], price)
+
     def add_storage_sku(self, sku: Sku) -> None:
         if sku.description.lower().startswith("ssd backed local storage"):
-            price = sku.pricing_info[0].pricing_expression.tiered_rates[0].unit_price
-            price = price.units + price.nanos / 1e9 / hours_in_month
+            price = self._calculate_sku_price(sku) / hours_in_month
             self._add_price(sku, self.local_ssd, price)
 
     @staticmethod
-    def _add_price(sku: Sku, family_prices: PricePerRegionSpot, price: float) -> None:
-        spot = sku.category.usage_type == "Preemptible"
-        for region in sku.service_regions:
-            family_prices[(region, spot)] = price
+    def _calculate_sku_price(sku: Sku) -> float:
+        price = sku.pricing_info[0].pricing_expression.tiered_rates[0].unit_price
+        return price.units + price.nanos / 1e9
 
-    def get_instance_price(self, instance: RawCatalogItem, spot: bool) -> Optional[float]:
+    @staticmethod
+    def _add_price(sku: Sku, family_prices: PricePerRegionCapacityType, price: float) -> None:
+        if sku.category.usage_type == "Preemptible":
+            capacity_type = CapacityType.SPOT
+        elif "DWS Calendar Mode" in sku.description:
+            capacity_type = CapacityType.DWS_CALENDAR_MODE
+        else:
+            capacity_type = CapacityType.ON_DEMAND
+        for region in sku.service_regions:
+            family_prices[(region, capacity_type)] = price
+
+    def get_instance_price(
+        self, instance: RawCatalogItem, capacity_type: CapacityType
+    ) -> Optional[float]:
         vm_family = self.get_vm_family(instance.instance_name)
         if vm_family in ["g1", "f1", "m2"]:  # shared-core and reservation-only
             return None
 
-        region_spot = (instance.location[:-2], spot)
-        if region_spot not in self.cpu[vm_family]:
-            return None
+        region_capacity_type = (instance.location[:-2], capacity_type)
 
+        # For some instances, the price is proportional to the number of GPUs
+        if instance.gpu_name and region_capacity_type in self.gpu_slice[instance.gpu_name]:
+            return instance.gpu_count * self.gpu_slice[instance.gpu_name][region_capacity_type]
+
+        # For others, the price consists of several components
         price = 0
-        price += instance.cpu * self.cpu[vm_family][region_spot]
-        price += instance.memory * self.ram[vm_family][region_spot]
+        if (
+            region_capacity_type not in self.cpu[vm_family]
+            or region_capacity_type not in self.ram[vm_family]
+        ):
+            return None
+        price += instance.cpu * self.cpu[vm_family][region_capacity_type]
+        price += instance.memory * self.ram[vm_family][region_capacity_type]
         if instance.gpu_name:
-            if region_spot not in self.gpu[instance.gpu_name]:
+            if region_capacity_type not in self.gpu[instance.gpu_name]:
                 return None
-            price += instance.gpu_count * self.gpu[instance.gpu_name][region_spot]
+            price += instance.gpu_count * self.gpu[instance.gpu_name][region_capacity_type]
         if instance.instance_name in local_ssd_sizes_gib:
-            price += local_ssd_sizes_gib[instance.instance_name] * self.local_ssd[region_spot]
+            price += (
+                local_ssd_sizes_gib[instance.instance_name] * self.local_ssd[region_capacity_type]
+            )
 
         return price
 
@@ -393,6 +489,35 @@ class Prices:
         return instance_name.split("-")[0]
 
 
+def set_flags(catalog_items: list[RawCatalogItem]) -> None:
+    for item in catalog_items:
+        if cast(GCPCatalogItemProviderData, item.provider_data).get("is_dws_calendar_mode"):
+            item.flags.append("gcp-dws-calendar-mode")
+        if item.instance_name.startswith("a4-"):
+            item.flags.append("gcp-a4")
+        elif item.instance_name.startswith("g4-standard-"):
+            item.flags.append("gcp-g4")
+
+
+# TODO: drop when dstack 0.19.33 is no longer relevant
+def add_legacy_g4_preview(catalog_items: list[RawCatalogItem]) -> list[RawCatalogItem]:
+    """
+    For each g4-standard-* instance, add a duplicate item with the "gcp-g4-preview" flag.
+
+    This is only needed for dstack 0.19.33, where the flag "gcp-g4-preview"
+    is used instead of "gcp-g4".
+    """
+    new_items = []
+    for item in catalog_items:
+        new_items.append(item)
+        if item.instance_name.startswith("g4-standard-"):
+            preview_item = copy.deepcopy(item)
+            preview_item.flags.remove("gcp-g4")
+            preview_item.flags.append("gcp-g4-preview")
+            new_items.append(preview_item)
+    return new_items
+
+
 def get_tpu_offers(project_id: str) -> list[RawCatalogItem]:
     logger.info("Fetching TPU offers")
     raw_catalog_items: list[RawCatalogItem] = []
@@ -599,23 +724,36 @@ def find_base_price_v5(
 
 
 def get_tpu_configs(project_id: str) -> list[dict]:
-    instances: list[dict] = []
-    client = tpu_v2.TpuClient()
-    for location in get_locations(project_id):
-        parent = f"projects/{project_id}/locations/{location}"
+    def _list_zone_configs(zone: str) -> list[dict]:
+        zone_instances = []
+        if zone in ["us-east1-b"]:
+            # These zones return
+            # google.api_core.exceptions.ServiceUnavailable: 503 502:Bad Gateway
+            return []
+        parent = f"projects/{project_id}/locations/{zone}"
         request = tpu_v2.ListAcceleratorTypesRequest(
             parent=parent,
         )
         page_result = client.list_accelerator_types(request=request)
         for response in page_result:
             no_of_chips = get_no_of_chips(response.accelerator_configs[0].topology)
-            instances.append(
+            zone_instances.append(
                 {
                     "instance_name": response.type_,
-                    "location": location,
+                    "location": zone,
                     "no_of_chips": no_of_chips,
                 }
             )
+        return zone_instances
+
+    client = tpu_v2.TpuClient()
+    instances: list[dict] = []
+    futures = []
+    with ThreadPoolExecutor(max_workers=8) as ex:
+        for zone in get_locations(project_id):
+            futures.append(ex.submit(_list_zone_configs, zone))
+    for future in as_completed(futures):
+        instances.extend(future.result())
     return instances
 
 
