@@ -1,12 +1,13 @@
 import copy
 import logging
 from concurrent.futures import ThreadPoolExecutor
-from typing import Optional
+from typing import Optional, cast
 
 import requests
 from requests import RequestException
+from typing_extensions import NotRequired, TypedDict
 
-from gpuhunt._internal.constraints import KNOWN_AMD_GPUS
+from gpuhunt._internal.constraints import find_accelerators
 from gpuhunt._internal.models import AcceleratorVendor, QueryFilter, RawCatalogItem
 from gpuhunt.providers import AbstractProvider
 
@@ -14,6 +15,12 @@ logger = logging.getLogger(__name__)
 API_URL = "https://api.runpod.io/graphql"
 
 
+class RunpodCatalogItemProviderData(TypedDict):
+    # `pod_counts` is the number of pods that can be deployed in a cluster for the given offer.
+    # Used to distinguish Runpod Clusters offers and check if multinode runs fit into clusters.
+    pod_counts: NotRequired[list[int]]
+
+
 class RunpodProvider(AbstractProvider):
     NAME = "runpod"
 
@@ -26,7 +33,6 @@ class RunpodProvider(AbstractProvider):
     @staticmethod
     def fetch_offers() -> list[RawCatalogItem]:
         query_variables = build_query_variables()
-
         with ThreadPoolExecutor(max_workers=10) as executor:
             futures = [
                 executor.submit(get_pods, query_variable) for query_variable in query_variables
@@ -42,6 +48,9 @@ class RunpodProvider(AbstractProvider):
         for query_variable, pods in zip(query_variables, pods_by_query):
             for pod in pods:
                 catalog_items.extend(make_catalog_items(query_variable, pod))
+
+        cluster_catalog_items = fetch_cluster_offers()
+        catalog_items.extend(cluster_catalog_items)
         return catalog_items
 
     @classmethod
@@ -75,53 +84,65 @@ def build_query_variables() -> list[dict]:
         for dc_id in data_centers:
             variables.append(
                 {
-                    "secureCloud": True,
-                    "dataCenterId": dc_id,
-                    "gpuCount": gpu_count,
-                    "minDisk": None,
-                    "minMemoryInGb": None,
-                    "minVcpuCount": None,
+                    "GpuTypeFilter": {
+                        "cluster": False,
+                    },
+                    "lowestPriceInput": {
+                        "secureCloud": True,
+                        "dataCenterId": dc_id,
+                        "gpuCount": gpu_count,
+                        "minDisk": None,
+                        "minMemoryInGb": None,
+                        "minVcpuCount": None,
+                    },
                 }
             )
         # Community cloud is queryable by country code
         for country_code in gpu_types["data"]["countryCodes"]:
+            if country_code is None:
+                continue
             variables.append(
                 {
-                    "secureCloud": False,
-                    "countryCode": country_code,
-                    "gpuCount": gpu_count,
-                    "minDisk": None,
-                    "minMemoryInGb": None,
-                    "minVcpuCount": None,
+                    "GpuTypeFilter": {
+                        "cluster": False,
+                    },
+                    "lowestPriceInput": {
+                        "secureCloud": False,
+                        "countryCode": country_code,
+                        "gpuCount": gpu_count,
+                        "minDisk": None,
+                        "minMemoryInGb": None,
+                        "minVcpuCount": None,
+                    },
                 }
             )
-
     return variables
 
 
-def get_pods(query_variable: dict) -> list[dict]:
+def get_pods(query_variables: dict) -> list[dict]:
     resp = make_request(
         {
             "query": query_pod_types,
-            "variables": {"lowestPriceInput": query_variable},
+            "variables": query_variables,
         }
     )
     return resp["data"]["gpuTypes"]
 
 
-def make_catalog_items(query_variable: dict, pod: dict) -> list[RawCatalogItem]:
+def make_catalog_items(query_variables: dict, pod: dict) -> list[RawCatalogItem]:
+    lowest_price_input_variables = query_variables["lowestPriceInput"]
     if pod["lowestPrice"]["stockStatus"] is None:
         return []
     listed_gpu_vendor_and_name = gpu_vendor_and_name(pod["id"])
     if listed_gpu_vendor_and_name is None:
         logger.warning(f"{pod['id']} missing in runpod GPU_MAP")
         return []
-    if query_variable["secureCloud"]:
-        location = query_variable["dataCenterId"]
+    if lowest_price_input_variables["secureCloud"]:
+        location = lowest_price_input_variables["dataCenterId"]
         on_demand_gpu_price = pod["securePrice"]
         spot_gpu_price = pod["secureSpotPrice"]
     else:
-        location = query_variable["countryCode"]
+        location = lowest_price_input_variables["countryCode"]
         on_demand_gpu_price = pod["communityPrice"]
         spot_gpu_price = pod["communitySpotPrice"]
     item_template = RawCatalogItem(
@@ -131,11 +152,12 @@ def make_catalog_items(query_variable: dict, pod: dict) -> list[RawCatalogItem]:
         cpu=pod["lowestPrice"]["minVcpu"],
         memory=pod["lowestPrice"]["minMemory"],
         gpu_vendor=listed_gpu_vendor_and_name[0],
-        gpu_count=query_variable["gpuCount"],
+        gpu_count=lowest_price_input_variables["gpuCount"],
         gpu_name=listed_gpu_vendor_and_name[1],
         gpu_memory=pod["memoryInGb"],
         spot=None,  # set below
         disk_size=None,
+        provider_data={},
     )
     items = []
     if on_demand_gpu_price:
@@ -151,6 +173,57 @@ def make_catalog_items(query_variable: dict, pod: dict) -> list[RawCatalogItem]:
     return items
 
 
+def fetch_cluster_offers() -> list[RawCatalogItem]:
+    cluster_catalog_items = []
+    query_variables = {
+        "gpuTypesInput": {
+            "cluster": True,
+        },
+        "lowestPriceInput": {
+            "gpuCount": 8,  # Needed to get CPU and RAM
+        },
+    }
+    pod_type = get_pods(query_variables)
+    for pod_type in pod_type:
+        listed_gpu_vendor_and_name = gpu_vendor_and_name(pod_type["id"])
+        if listed_gpu_vendor_and_name is None:
+            logger.warning(f"{pod_type['id']} missing in runpod GPU_MAP")
+            continue
+        gpu_vendor, gpu_name = listed_gpu_vendor_and_name
+        # Runpod returns no CPU and memory if the offer is out of stock.
+        # Out of stock appears to be different from no capacity meaning
+        # the offer is not available for a prolonged period.
+        cpu = pod_type["lowestPrice"].get("minVcpu")
+        memory = pod_type["lowestPrice"].get("minMemory")
+        if cpu is None:
+            logger.warning(f"{pod_type['id']} cluster offer missing minVcpu")
+            continue
+        if memory is None:
+            logger.warning(f"{pod_type['id']} cluster offer missing minMemory")
+            continue
+        for location in pod_type["nodeGroupDatacenters"]:
+            catalog_item = RawCatalogItem(
+                instance_name=pod_type["id"],
+                location=location["id"],
+                price=pod_type["clusterPrice"] * pod_type["maxGpuCount"],
+                cpu=cpu,
+                memory=memory,
+                gpu_vendor=gpu_vendor,
+                gpu_count=pod_type["maxGpuCount"],
+                gpu_name=gpu_name,
+                gpu_memory=pod_type["memoryInGb"],
+                spot=False,
+                disk_size=None,
+                flags=["runpod-cluster"],
+                # The API does not return supported pod counts but for now it's always 2-8 for all offers.
+                provider_data=cast(
+                    dict, RunpodCatalogItemProviderData(pod_counts=list(range(2, 9)))
+                ),
+            )
+            cluster_catalog_items.append(catalog_item)
+    return cluster_catalog_items
+
+
 def make_request(payload: dict):
     resp = requests.post(API_URL, json=payload, timeout=10)
     resp.raise_for_status()
@@ -183,6 +256,8 @@ def get_gpu_name(vendor: AcceleratorVendor, name: str) -> Optional[str]:
 
 
 def get_nvidia_gpu_name(name: str) -> Optional[str]:
+    if "B200" in name:
+        return "B200"
     if "V100" in name:
         return "V100"
     if name == "H100 NVL":
@@ -198,9 +273,8 @@ def get_nvidia_gpu_name(name: str) -> Optional[str]:
 
 
 def get_amd_gpu_name(name: str) -> Optional[str]:
-    for gpu in KNOWN_AMD_GPUS:
-        if gpu.name == name:
-            return name
+    if accelerators := find_accelerators(names=[name], vendors=[AcceleratorVendor.AMD]):
+        return accelerators[0].name
     return None
 
 
@@ -246,6 +320,10 @@ query GpuTypes($lowestPriceInput: GpuLowestPriceInput, $gpuTypesInput: GpuTypeFi
     }
     maxGpuCount
     id
+    nodeGroupDatacenters {
+        id
+    }
+    clusterPrice
     displayName
     memoryInGb
     securePrice
