@@ -4,6 +4,8 @@ from pathlib import Path
 
 import pytest
 
+from gpuhunt.providers.cloudrift import GPU_MAP
+
 
 @pytest.fixture
 def data_rows(catalog_dir: Path) -> list[dict]:
@@ -11,10 +13,17 @@ def data_rows(catalog_dir: Path) -> list[dict]:
         return list(csv.DictReader(f))
 
 
-# TODO: Add RTX5090 and RTX6000PRO and others after evaluation
-@pytest.mark.parametrize("gpu", ["RTX4090"])
-def test_gpu_present(gpu: str, data_rows: list[dict]):
-    assert gpu in map(itemgetter("gpu_name"), data_rows)
+def select_row(rows, name: str) -> list[str]:
+    return [r[name] for r in rows if r[name]]
+
+
+def test_gpu_present(data_rows: list[dict]):
+    expected_gpus = [gpu for _, gpu in GPU_MAP]
+    gpus = select_row(data_rows, "gpu_name")
+    gpus = list(dict.fromkeys(gpus))
+    assert set(gpus).issubset(
+        set(expected_gpus)
+    ), f"Found unexpected GPUs: {set(gpus) - set(expected_gpus)}"
 
 
 # TODO: Add 3, 4, 5, ... 8
@@ -26,7 +35,3 @@ def test_gpu_count_present(gpu_count: int, data_rows: list[dict]):
 @pytest.mark.parametrize("location", ["us-east-nc-nr-1"])
 def test_location_is_present(location: str, data_rows: list[dict]):
     assert location in map(itemgetter("location"), data_rows)
-
-
-def test_non_zero_price(data_rows: list[dict]):
-    assert all(float(p) > 0 for p in map(itemgetter("price"), data_rows))
