@@ -33,7 +33,6 @@ def test_locations(data_rows: list[dict]):
         "europe-central-1",
         "me-west-1",
         "us-east-1",
-        "us-east-2",
         "us-east-3",
         "us-midwest-1",
         "us-south-1",
@@ -46,7 +45,3 @@ def test_locations(data_rows: list[dict]):
     locations = set(map(itemgetter("location"), data_rows))
     missing = expected_locations - locations
     assert not missing
-
-
-def test_non_zero_price(data_rows: list[dict]):
-    assert all(float(p) > 0 for p in map(itemgetter("price"), data_rows))
